#pragma once

#include "Database.h"

namespace hyplex::drakan
{
	struct Sequence : INode
	{
		enum class ERelativeTo
		{
			THE_WORLD,
			ANOTHER_ACTOR,
			LOOKAT_ACTOR,
		};

		enum class EInterpolation
		{
			NONE,
			LINEAR_LINEAR,
			LINEAR_SPLINE,
			SINE_SPLINE,
		};

		struct XFormKeyframe
		{
			fquat rotation{};
			fvec3 translation{};
			bool ignorePosition = false;
			bool ignoreRotation = false;
			ERelativeTo relative{};
			EInterpolation interpolation{};
		};

		struct SoundAction
		{
			uint32_t soundId = 0;
			float duration = 0;
			float volume = 0;
			bool mono = false;
			bool notifyActor = false; // ??
		};

		struct RunStopAI
		{
			bool runAI = false;
		};

		enum class ERootNodeTranslationMode
		{
			MESH, FRAME, NONE, INVALID
		};

		struct AnimationAction
		{
			unsigned int channelI = 0;
			uint32_t animId = 0;
			float transitionTime = 0;
			float speed = 0;

			// Iff channelI == 0.
			ERootNodeTranslationMode rootNodeTranslationMode[3]{};
		};

		struct AttachActorAction
		{
			uint32_t attachedActorId = 0;
			uint32_t thisActorsChannelId = 0;
		};

		// Probably used to trigger effects.
		struct MessageAction
		{
			uint32_t message = 0;
		};

		struct ShowHideAction
		{
			bool visible = false;
		};

		struct Action
		{
			float t = 0;
			std::variant<std::monostate, XFormKeyframe, SoundAction, RunStopAI, AnimationAction, AttachActorAction, MessageAction,
				ShowHideAction> info;
		};

		struct Actor
		{
			unsigned int id = 0;
			uint32_t levelEntityId = 0;
			String name;
			std::vector<Action> actions;
		};

		unsigned int id = 0;
		String name;

		std::vector<Actor> actors;

		void consumeData(const Entry& entry, BasicBufferedLexState& bin);
		virtual ENodeType getType() const noexcept override;
		virtual void dumpNode(const NodeDumpArg& arg) const override;
	};
}
